# Copyright 2020 Unlimited Robotics

import os
import sys
import subprocess
import glob

from rayasdk.logger import log_error, log_verbose

# Constants
CUR_USER_HOME = os.path.expanduser('~')
PLATFORM = sys.platform

if PLATFORM == 'windows':
    # https://stackoverflow.com/questions/23064052/how-to-modify-ssh-folder-files-in-windows
    # todo implement this on ws
    PRIV_SSH_DIR = f'{CUR_USER_HOME}/.ssh'
elif PLATFORM == 'linux' or PLATFORM == 'darwin':
    PRIV_SSH_DIR = f'{CUR_USER_HOME}/.ssh'


class SshKeyGen:

    def __init__(self, user, host, port=22):
        self.return_code = -1
        if port != 22:
            self.push_key(user, host, port)
        else:
            self.push_key(user, host)

    def key_present(self):
        """Checks to see if there is an key already present. Returns a bool."""
        search_pattern = 'id_*[!*.pub]'
        matching_files = glob.glob(os.path.join(PRIV_SSH_DIR, search_pattern))
        if matching_files:
            return True
        else:
            return False

    def gen_key(self):
        """Generate a SSH Key."""
        os.chdir(PRIV_SSH_DIR)
        if self.key_present():
            log_verbose('A key is already present.')
            return False
        else:
            # Generate private key
            if subprocess.call('ssh-keygen', shell=True) != 0:
                log_error('The ssh-keygen command could not create the key'
                          'try to generate this key manually')
                return False
            return True

    def push_key(self, user, host, port=22):
        """Push an SSH Key to a remote server."""
        os.chdir(PRIV_SSH_DIR)
        if not self.key_present():
            self.gen_key()

        if PLATFORM == 'linux':
            if 'ssh-copy-id' in os.listdir('/usr/local/bin'):
                log_verbose('SSH key found. Pushing key to remote server')
                command = f'ssh-copy-id -o StrictHostKeyChecking=no '
                command += f'-p {port} {user}@{host}'
                ssh_copy_process = subprocess.Popen(command,
                                                    shell=True,
                                                    stdin=subprocess.PIPE,
                                                    stdout=subprocess.PIPE,
                                                    stderr=subprocess.PIPE)
                output, err = ssh_copy_process.communicate()
                if ssh_copy_process.returncode != 0:
                    log_error(err.decode())
                    raise subprocess.SubprocessError
        else:
            log_error('Currently this only works on linux')
