import os
import json
import progressbar

import rayasdk.constants as constants
from rayasdk.messages import *
from rayasdk.container_handlers.docker_handler import DockerHandler


class MyProgressBar:

    def __init__(self):
        self.pbar = None

    def __call__(self, block_num, block_size, total_size):
        if self.pbar is None:
            self.pbar = progressbar.ProgressBar(maxval=total_size)
            self.pbar.start()

        downloaded = block_num * block_size
        if downloaded < total_size:
            self.pbar.update(downloaded)
        else:
            self.pbar.finish()


def run_simulator_bridge():
    return DockerHandler().launch_command()


def is_snake_case(s: str):
    """
    Returns True if s is in snake case, False otherwise.
    """
    if not s.islower():
        return False
    if '__' in s:
        return False
    if s.startswith('_') or s.endswith('_'):
        return False
    if not all(c.isalpha() or c.isdigit() or c == '_' for c in s):
        return False
    return True


def validate_app_id_and_folder_name(app_id: str):
    APP_MAIN_FOLDER_NAME = os.path.basename(os.path.normpath(constants.CURRENT_PATH))
    if APP_MAIN_FOLDER_NAME != app_id:
        raise Exception('The app_id and the folder name are different')
    if not is_snake_case(app_id):
        raise Exception('The app_id have to be in snake case format')
    return True


def open_connection_file():
    try:
        with open(constants.CONNECTION_SETTINGS_PATH, 'r', encoding='utf-8') as f:
            connection_settings = json.load(f)
        return connection_settings
    except OSError:
        raise Exception(
            (f'the file "{constants.CONNECTION_SETTINGS_FILE}" was not found, '
             'run \'rayasdk connect\''))
